@@ -34,6 +34,7 @@ class FinalFeedback:
         self.success = success
         self.score = score
         self._scores = []
+        self._scores_feedback = []
         self.category = category
         self.label = label
         self.title = title
@@ -45,6 +46,8 @@ class FinalFeedback:
         self.systems = []
         self.suppressions = suppressions
         self.suppressed_labels = suppressed_labels
+        self.considered = []
+        self.used = []
 
     def merge(self, feedback):
         category = feedback.category.lower()
@@ -66,12 +69,12 @@ class FinalFeedback:
             self._scores.append(f"{inversion}{partial}")
         if not feedback and feedback.else_message:
             self.positives.append(feedback)
-            return
+            return feedback
         if not feedback or feedback.muted:
             return
         if feedback.kind == Feedback.KINDS.COMPLIMENT:
             self.positives.append(feedback)
-            return
+            return feedback
         if feedback.kind == Feedback.KINDS.INSTRUCTIONAL:
             self.instructions.append(feedback)
         self.success = success and self.success
@@ -81,6 +84,8 @@ class FinalFeedback:
             self.category = feedback.category
             self.label = feedback.label
             self.data = data
+            self.used.append(feedback)
+        return feedback
 
     def finalize(self):
         if self.message is None:
@@ -127,4 +132,4 @@ class FinalFeedback:
             'message': self.message,
             'data': self.data,
             'hide_correctness': self.hide_correctness
-        }
\ No newline at end of file
+        }
