@@ -5,6 +5,7 @@ behavior.
 import re
 import types
 
+from pedal.core.report import Report
 from pedal.sandbox.exceptions import (SandboxNoMoreInputsException,
                                       SandboxPreventModule)
 from pedal.utilities.system import IS_PYTHON_36
@@ -74,25 +75,49 @@ def disabled_builtin(name):
 _OPEN_FORBIDDEN_NAMES = re.compile(r"(^[./])|(\.py$)")
 _OPEN_FORBIDDEN_MODES = re.compile(r"[wa+]")
 
-# TODO: Turn this into a function that lets us more elegantly specify valid and
-# invalid filenames/paths
+# TODO: Allow the user to give a function instead of a REGEX
+# TODO: We need to mock the whole OS library, honestly, not just `open`
+def create_open_function(report: Report, forbidden_names=_OPEN_FORBIDDEN_NAMES, forbidden_modes=_OPEN_FORBIDDEN_MODES):
+    """
+    Creates a new version of the `open` built-in that will avoid looking at certain filenames,
+    and will also respect the given Report's Submission's Files.
 
+    Args:
+        report:
 
-def _restricted_open(name, mode='r', buffering=-1):
-    if _OPEN_FORBIDDEN_NAMES.search(name):
-        raise RuntimeError("The filename you passed to 'open' is restricted.")
-    elif _OPEN_FORBIDDEN_MODES.search(mode):
-        raise RuntimeError("You are not allowed to 'open' files for writing.")
-    else:
-        return ORIGINAL_BUILTINS['open'](name, mode, buffering)
+    Returns:
+
+    """
+    def _restricted_open(name, mode='r', buffering=-1):
+        if forbidden_names.search(name):
+            raise RuntimeError("The filename you passed to 'open' is restricted.")
+        elif forbidden_modes.search(mode):
+            raise RuntimeError("You are not allowed to 'open' files for writing.")
+        elif report.submission and name in report.submission.files:
+            return report.submission.files[name]
+        else:
+            return ORIGINAL_BUILTINS['open'](name, mode, buffering)
+    return _restricted_open
 
 # TODO: Allow this to be flexible
 
 
-def _restricted_import(name, globals=None, locals=None, fromlist=(), level=0):
-    if name == 'pedal' or name.startswith('pedal.'):
-        raise RuntimeError("You cannot import pedal!")
-    return ORIGINAL_BUILTINS['__import__'](name, globals, locals, fromlist, level)
+def create_import_function(report: Report):
+    """
+    Creates a new version of the `__import__` function (which is used by the import keyword)
+    that will not let students import Pedal or its submodules.
+
+    Args:
+        report:
+
+    Returns:
+
+    """
+    def _restricted_import(name, globals=None, locals=None, fromlist=(), level=0):
+        if name == 'pedal' or name.startswith('pedal.'):
+            raise RuntimeError("You cannot import pedal!")
+        return ORIGINAL_BUILTINS['__import__'](name, globals, locals, fromlist, level)
+    return _restricted_import
 
 
 try:
@@ -117,6 +142,7 @@ ORIGINAL_BUILTINS = {
     'exec': _default_builtins.get('exec', _disabled_exec),
     'eval': _default_builtins.get('eval', _disabled_eval),
     'compile': _default_builtins.get('compile', _disabled_compile),
+    'exit': disabled_builtin('exit'),
     '__import__': _default_builtins['__import__']
 }
 
@@ -331,30 +357,35 @@ class MockPlt(MockModule):
     def _reset_plot(self):
         self.active_plot = {'data': [],
                             'xlabel': None, 'ylabel': None,
-                            'title': None, 'legend': False}
+                            'title': None, 'legend': False,
+                            'kwargs': {},
+                            'xlabel_kwargs': {}, 'ylabel_kwargs': {},
+                            'title_kwargs': {}, 'suptitle_kwargs': {},
+                            'legend_kwargs': {},
+                            'other_calls': []}
 
     def hist(self, data, **kwargs):
         """ Make a histogram """
         label = kwargs.get('label', None)
         self.active_plot['data'].append({'type': 'hist', 'values': data,
-                                         'label': label})
+                                         'label': label, 'kwargs': kwargs})
 
     def boxplot(self, data, **kwargs):
         label = kwargs.get('label', None)
         self.active_plot['data'].append({'type': 'boxplot', 'values': data,
-                                         'label': label})
+                                         'label': label, 'kwargs': kwargs})
 
     def hlines(self, y, xmin, xmax, **kwargs):
         label = kwargs.get('label')
         self.active_plot['data'].append({'type': 'hlines', 'y': y,
                                          'xmin': xmin, 'xmax': xmax,
-                                         'label': label})
+                                         'label': label, 'kwargs': kwargs})
 
     def vlines(self, x, ymin, ymax, **kwargs):
         label = kwargs.get('label')
         self.active_plot['data'].append({'type': 'vlines', 'x': x,
                                          'ymin': ymin, 'ymax': ymax,
-                                         'label': label})
+                                         'label': label, 'kwargs': kwargs})
 
     def plot(self, xs, ys=None, **kwargs):
         """ Make a line plot """
@@ -362,46 +393,57 @@ class MockPlt(MockModule):
         if ys is None:
             self.active_plot['data'].append({'type': 'line',
                                              'x': list(range(len(xs))),
-                                             'y': xs, 'label': label})
+                                             'y': xs, 'label': label,
+                                             'kwargs': kwargs})
         else:
             self.active_plot['data'].append({'type': 'line', 'x': xs,
-                                             'y': ys, 'label': label})
+                                             'y': ys, 'label': label,
+                                             'kwargs': kwargs})
 
     def scatter(self, xs, ys, **kwargs):
         """ Make a scatter plot """
         label = kwargs.get('label', None)
         self.active_plot['data'].append({'type': 'scatter', 'x': xs,
-                                         'y': ys, 'label': label})
+                                         'y': ys, 'label': label,
+                                         'kwargs': kwargs})
 
     def bar(self, xs, ys, **kwargs):
-        """ Make a scatter plot """
+        """ Make a bar plot """
         label = kwargs.get('label', None)
         self.active_plot['data'].append({'type': 'bar', 'x': xs,
-                                         'y': ys, 'label': label})
+                                         'y': ys, 'label': label,
+                                         'kwargs': kwargs})
 
     def xlabel(self, label, **kwargs):
         """ Label the x-axis """
         self.active_plot['xlabel'] = label
+        self.active_plot['xlabel_kwargs'].update(kwargs)
 
     def title(self, label, **kwargs):
         """ Make the title """
         self.active_plot['title'] = label
+        self.active_plot['title_kwargs'].update(kwargs)
 
     def suptitle(self, label, **kwargs):
         """ Make the super title """
         self.title(label, **kwargs)
+        self.active_plot['suptitle_kwargs'].update(kwargs)
 
     def ylabel(self, label, **kwargs):
         """ Label the Y-axis """
         self.active_plot['ylabel'] = label
+        self.active_plot['ylabel_kwargs'].update(kwargs)
 
     def legend(self, **kwargs):
         """ Show the legend """
         self.active_plot['legend'] = True
+        self.active_plot['legend_kwargs'].update(kwargs)
 
     def _generate_patches(self):
         def dummy(*args, **kwargs):
             """ This function does nothing. """
+            # TODO: Capture name; does this need to be a decorated function?
+            self.active_plot['other_calls'].append((args, kwargs))
 
         return dict(hist=self.hist, plot=self.plot,
                     bar=self.bar, boxplot=self.boxplot,
