@@ -61,7 +61,7 @@ class SqlProgSnap2(BaseProgSnap2):
     PROFILES = {
         'blockpy': dict(link_filters={
             'Subject': {
-                'X-IsStaff': "False",
+                'X-IsStaff': "",
             },
         },
             link_selections={
@@ -90,7 +90,7 @@ class SqlProgSnap2(BaseProgSnap2):
         self.profile = None
         self.cache = cache
         # To see executed queries:
-        # self._connection.set_trace_callback(print)
+        self._connection.set_trace_callback(print)
         if status_update:
             self._connection.set_progress_handler(status_update, 1000)
 
